@@ -3,14 +3,12 @@
 """
 Directory : orpyste
 Name      : reader
-Version   : 2013.04
+Version   : 2013.05
 Author    : Christophe BAL
 Mail      : projetmbc@gmail.com
 
-This module allows  to read efficient to type and simple structured datas using
-in files by using the ``peuf`` specifications.
-
-See the documentation for more details.
+This module allows to read efficient to type and simple structured string datas
+contained in files using the ``peuf`` specifications.
 """
 
 import collections
@@ -24,6 +22,13 @@ from orpyste import model
 # ------------------------- #
 
 class ReaderError(ValueError):
+    """
+-----------------
+Small description
+-----------------
+
+Base class for errors in the ``reader`` module of the package ``orpyste``.
+    """
     pass
 
 
@@ -32,6 +37,13 @@ class ReaderError(ValueError):
 # --------------------------- #
 
 def DEFAULT_IS_BLOCK_NAME(text):
+    """
+-----------------
+Small description
+-----------------
+
+This function tests that the string ``text`` is a legal name for a ¨peuf block.
+    """
     return regex_use.isVarName(
         text = text,
         kind = "lazzy"
@@ -44,6 +56,10 @@ def DEFAULT_IS_BLOCK_NAME(text):
 
 class Build(model.parser.Skeleton):
     """
+-----------------
+Small description
+-----------------
+
 Suppose that you have the following ``orpyste`` file (sorry for the lake of
 originality).
 
@@ -100,7 +116,7 @@ dictionary.
 
 python::
     {
-        'games_rules'    : "Play !\nTry to win.\nWin or let go.",
+        'games_rules'    : "Play !\\nTry to win.\\nWin or let go.",
         'names_of_levels': ["easy", "medium", "hard"],
         'player_1': {
             'age'  : '18',
@@ -118,7 +134,7 @@ python::
 
 
 You can also work directly with string like in the following example which has
-the same effect as the pre ceding one.
+the same effect as the preceding one.
 
 python::
     import orpyste
@@ -157,14 +173,14 @@ python::
     )
 
 
-One last thing, if you want to have a finer control for the block and line
-contents, just use the attribut ``dictNbLine``. For the preceding code,
-``infos.dictNbLine`` is equal to following ordered dictionary where the content
-are replaced by uplets ``(Number of the line, Content)``.
+If you want to have a finer control for the block and line contents, just use
+the attribut ``dictNbLine``. For the preceding code, ``infos.dictNbLine`` is
+equal to following ordered dictionary where the content are replaced by uplets
+``(Number of the line, Content)``.
 
 python::
     {
-        'game_rules'     : (3, 'Play !\nTry to win.\nWin or let go.'),
+        'game_rules'     : (3, 'Play !\\nTry to win.\\nWin or let go.'),
         'names_of_levels': [(8, 'easy'), (9, 'medium'), (10, 'hard')],
         'player_1': {
             'age'  : '18',
@@ -181,6 +197,52 @@ python::
     }
 
 
+Let finish with a very special mode with the following ``orpyste`` file where we
+want to use ``<==>``, ``==>``, ``=`` like ``key-value`` separators.
+
+orpyste::
+    logic::
+        a <==> b
+        x ==> y
+
+    constraint::
+        i     = j
+        alpha <= beta
+
+
+Let suppose that the preceding file has the path path::``user/myFile.txt``. Then
+we can extract informations by using the following code.
+
+python::
+    import orpyste
+
+    infos = orpyste.reader.Build(
+        path = "user/myFile.txt",
+        mode = "keyval",
+        sep  = ["<==>", "==>", "=", "<="]
+    )
+
+
+Using the preceding code, then ``infos.dict`` will be the following ordered
+dictionary.
+
+python::
+    {
+        'logic': {
+            '<==>': {'a': "b"},
+            '==>' : {'x': "y"}
+        },
+        'constraint': {
+            '=' : {'i': "j"},
+            '<=': {'alpha': "beta"}
+        }
+    }
+
+
+-------------
+The arguments
+-------------
+
 This class uses the following variables.
 
     1) ``content`` is one content to analyse. You can use this variable or the
@@ -201,27 +263,35 @@ This class uses the following variables.
 
         b) ``"equal+"`` is similar to ``"equal"`` except that newlines are
         allowed inside values. To use one sign ``=`` in one value, you have to
-        escape at least the first sign ``=`` via ``"="``.
+        escape at least the first sign ``=`` via ``\=``.
 
-        << WARNING ! >> Each newline will be translated to one single space.
+        warning::
+            Each empty line will be translated to one single space.
 
-        c) ``"line"`` is for informations stored line by line without any
+        c) ``"keyval"`` extends the mode ``"equal"`` by allowing to use
+        different separators defined in the variable ``sep`` (see below).
+
+        d) ``"keyval+"`` plays for ``"keyval"`` the same role that ``"equal+"``
+        does for ``"equal"``.
+
+        e) ``"line"`` is for informations stored line by line without any
         special syntax in each line in every blocks to analyse.
 
-        d) ``"block"`` is for information stored in a whole paragraph made of
+        f) ``"block"`` is for information stored in a whole paragraph made of
         several lines in every blocks to analyse.
 
-        e) We can also use one of the three preceding modes for different
-        blocks. In that case, we simply use one dictionary like in the following
-        example where the special mode ``"container"`` is for blocks that
-        contains other blocks. Indeed, you can just use the keys you want to
-        use.
+        g) We can also use one of the preceding modes for different blocks. In
+        that case, we simply use one dictionary like in the following example
+        where the special mode ``"container"`` is for blocks that contains other
+        blocks. Indeed, you can just use only the keys you need.
 
         python::
             mode = {
                 'default'  : the default mode,
                 'equal'    : [names of the blocks],
                 'equal+'   : [names of the blocks],
+                'keyval'   : [names of the blocks],
+                'keyval+'  : [names of the blocks],
                 'line'     : [names of the blocks],
                 'block'    : [names of the blocks],
                 'container': [names of the blocks]
@@ -233,23 +303,42 @@ This class uses the following variables.
         If you don't define the value associated to the key ``'default'`` then
         the value ``"container"`` will be use by default.
 
-    5) ``strip`` is a boolean variable to ask to strip the start of text
+    5) ``sep`` indicates which text(s) must be used to separate ``key`` and
+    ``value`` for the modes ``"equal"``, ``"equal+"``, ``"keyval"`` and
+    ``"keyval+"``. You can use a single string or a list of strings.
+
+    By default, ``sep = "="`` which is justified for the use of the classical
+    modes ``"equal"`` and ``"equal+"``.
+
+    warning::
+        You can't use a list of separators with one of the modes ``"equal"`` and
+        ``"equal+"``.
+
+    6) ``strip`` is a boolean variable to ask to strip the start of text
     containing informations for line or block contents. The default value is
     ``False``.
 
-    6) ``isBlockName`` is one method that must return one boolean. This method
+    7) ``isBlockName`` is one method that must return one boolean. This method
     is used to validate the name used for blocks.
 
     By default, ``isBlockName(text)`` is equal to ``regex_use.isVarName(text,
     kind = "lazzy")`` where ``regex_use`` is one module of the package ¨mistool.
     """
 
+    ALLOWED_MODE = [
+        "equal", "equal+",
+        "keyval", "keyval+",
+        "line",
+        "block"
+    ]
+
     def __init__(
         self,
         content     = "",
         path        = "",
         encoding    = "utf8",
         mode        = "equal",
+        sep         = "=",
         strip       = False,
         isBlockName = DEFAULT_IS_BLOCK_NAME
     ):
@@ -257,37 +346,12 @@ This class uses the following variables.
         self.content     = content
         self.encoding    = encoding
         self.mode        = mode
+        self.sep         = sep
         self.strip       = strip
         self.isBlockName = isBlockName
 
         self.build()
 
-# --------------------- #
-# -- FLAT DICTIONARY -- #
-# --------------------- #
-
-    def __flattenDict(self, oneDict):
-        """
-This method transforms the ``(Number line, Content)`` dictionary to a simple
-``Content`` dictionary.
-        """
-        flatDict = collections.OrderedDict()
-
-        for oneKey, oneValue in oneDict.items():
-            typeValue = type(oneValue)
-
-            if typeValue in [dict, collections.OrderedDict]:
-                oneValue = self.__flattenDict(oneValue)
-
-            elif typeValue == tuple:
-                oneValue = oneValue[1]
-
-            elif typeValue == list:
-                oneValue = [x[1] for x in oneValue]
-
-            flatDict[oneKey] = oneValue
-
-        return flatDict
 
 # ------------------------------- #
 # -- BUILDING THE DICTIONARIES -- #
@@ -295,6 +359,10 @@ This method transforms the ``(Number line, Content)`` dictionary to a simple
 
     def build(self):
         """
+-----------------
+Small description
+-----------------
+
 Each time you make an instance of ``Reader``this method is called so as to
 automatically build the ``(Number line, Content)`` and the simple ``Content``
 dictionaries.
@@ -303,7 +371,8 @@ You can use this method if you want to rebuild this dictionaries after a change
 of the content of the file or of the value of the attribut ``content``.
         """
 # Which mode is used ?
-        if type(self.mode) in [dict, collections.OrderedDict]:
+        if isinstance(self.mode, dict) \
+        or isinstance(self.mode, collections.OrderedDict):
             self.__singleMode__ = ":mix:"
             self.__mixedModes__ = self.mode
 
@@ -311,14 +380,30 @@ of the content of the file or of the value of the attribut ``content``.
                 self.__mixedModes__['default'] = "container"
 
         else:
-            if self.mode not in ["equal", "equal+", "line", "block"]:
+            if self.mode not in self.ALLOWED_MODE:
                 raise ReaderError(
-                    'Unknown mode "{0}".'.format(self.mode)
+                    'Unknown mode << {0} >>.'.format(self.mode)
                 )
 
             self.__singleMode__ = self.mode
             self.__mixedModes__ = None
 
+# Normalize ``sep``.
+        if isinstance(self.sep, str):
+            self.sep = [self.sep]
+
+        else:
+            self.sep = sorted(
+                self.sep,
+                key = lambda t: -len(t)
+            )
+
+        if self.mode in ["equal", "equal+"] and len(self.sep) !=1:
+            raise ReaderError(
+                'Only one separator is allowed for the mode '
+                '<< {0} >>.'.format(self.mode)
+            )
+
 # Which content must be analysed ?
         if self.path:
             content = os_use.readTextFile(self.path)
@@ -329,6 +414,9 @@ of the content of the file or of the value of the attribut ``content``.
         content = content.split('\n')
 
 # The (number line, content) dictionary.
+#
+# << Warning ! >> We keep the comments so as to have the right numbers of
+# the lines.
         self.dictNbLine = self.__analyse(
             mode        = self.__singleMode__,
             linesWithNb = [(n + 1, x) for n, x in enumerate(content)]
@@ -345,13 +433,18 @@ of the content of the file or of the value of the attribut ``content``.
         inBlock = False
     ):
         """
-This method does all the ugly job for analysing the ``orpyste`` file.
+-----------------
+Small description
+-----------------
+
+This method does all the ugly job for analysing the ``orpyste`` file so as to
+build the ``(number line, content)`` dictionary.
         """
-        oneBlockHasBeenFound = False
-        lastKeyPlus          = None
+        blockFound  = False
+        lastKeyPlus = None
 
         popNewLine = True
-        weAreInComment  = False
+        inComment  = False
 
         answer = collections.OrderedDict()
 
@@ -366,23 +459,26 @@ This method does all the ugly job for analysing the ``orpyste`` file.
             storeThisLine = (mode == "block" or not oneLine.strip() == '')
 
 
-# COMMENT ON SEVERAL LINES
-            if weAreInComment:
+# -- COMMENT ON SEVERAL LINES -- #
+
+            if inComment:
                 if self.isMultiLineCommentEnd(oneLine):
-                    weAreInComment = False
+                    inComment = False
 
             elif self.isMultiLineCommentStart(oneLine):
 # We can have  /*...*/  in the same line.
                 if not self.isMultiLineCommentEnd(oneLine):
-                    weAreInComment = True
+                    inComment = True
 
 
-# COMMENT IN ONE SINGLE LINE
+# -- COMMENT IN ONE SINGLE LINE -- #
+
             elif self.isOneLineComment(oneLine):
                 ...
 
 
-# BLOCK
+# -- BLOCK -- #
+
             elif self.isNewBlock(oneLine):
                 lastKeyPlus = None
 
@@ -393,8 +489,8 @@ This method does all the ugly job for analysing the ``orpyste`` file.
                         'Wrong indentation in the line {0}.'.format(nbLine)
                     )
 
-                oneBlockHasBeenFound = True
-                nameBlockFound       = oneLine.strip()[:-2]
+                blockFound     = True
+                nameBlockFound = oneLine.strip()[:-2]
 
 # Is the name of the block a legal one ?
                 if not self.isBlockName(nameBlockFound):
@@ -410,18 +506,18 @@ This method does all the ugly job for analysing the ``orpyste`` file.
                 blockContent = []
                 popNewLine   = False
 
-                weAreInCommentInContent = False
+                inCommentInContent = False
 
                 while(linesWithNb):
                     nbLine, oneLine = linesWithNb.pop(0)
 #    + Comment in the content of a block
-                    if weAreInCommentInContent:
+                    if inCommentInContent:
                         if self.isMultiLineCommentEnd(oneLine):
-                            weAreInCommentInContent = False
+                            inCommentInContent = False
 
                     elif self.isMultiLineCommentStart(oneLine):
                         if not self.isMultiLineCommentEnd(oneLine):
-                            weAreInCommentInContent = True
+                            inCommentInContent = True
 
 #    + Comment in one single line in the content of a block
                     elif self.isOneLineComment(oneLine):
@@ -466,56 +562,73 @@ This method does all the ugly job for analysing the ``orpyste`` file.
                 )
 
 
-# CONTENT TO STORE
+# -- CONTENT TO STORE -- #
+
             elif storeThisLine:
-                if not inBlock or oneBlockHasBeenFound:
+                if not inBlock or blockFound:
                     self.__sendError(
                         'Data and block have the same level '\
                         'of indentation. See the line {0}.'.format(nbLine)
                     )
 
 # Mode used --> key = value
-                if mode == "equal":
-                    i = oneLine.find('=')
+                if mode in ["equal", "keyval"]:
+                    iKey, iVal, sep = self.__findSep(oneLine)
 
-                    if i == -1:
+                    if iKey == -1:
                         self.__sendError(
                             'Missing equal sign in the line {0}.'\
                             .format(nbLine)
                         )
 
-                    answer[oneLine[:i].strip()] = oneLine[i + 1:].strip()
+                    self.__storeKeyval(
+                        answer = answer,
+                        key    = oneLine[:iKey].strip(),
+                        val    = oneLine[iVal:].strip(),
+                        sep    = sep,
+                        mode   = mode
+                    )
 
 # Mode used --> key = value with newlines
-                elif mode == "equal+":
-                    i = self.findEqualPlus(oneLine)
+                elif mode in ["equal+", "keyval+"]:
+                    iKey, iVal, sep = self.__findSep(oneLine)
 
-                    if i == -1:
+                    if iKey == -1:
                         if lastKeyPlus == None:
                             self.__sendError(
                                 'Missing equal sign in the line {0}.'\
                                 .format(nbLine)
                             )
 
-                        answer[lastKeyPlus] \
-                        += ' ' + oneLine.replace('"="', '=').strip()
+                        answer[lastKeyPlus] += ' ' + self.__cleanEscape(
+                            text         = oneLine.strip(),
+                            escapeKeyval = True
+                        )
 
                     else:
-                        lastKeyPlus = oneLine[:i].strip()
+                        lastKeyPlus = oneLine[:iKey].strip()
 
-                        answer[lastKeyPlus] \
-                        = oneLine[i + 1:].replace('"="', '=').strip()
+                        self.__storeKeyval(
+                            answer = answer,
+                            key    = lastKeyPlus,
+                            val    = oneLine[iVal:].strip(),
+                            sep    = sep,
+                            mode   = mode
+                        )
 
 # Simply one content to finish to store
                 elif mode in ["line", "block", ":mix:"]:
-                    answer[nbLine] = self.stripIt(oneLine)
+                    answer[nbLine] = self.stripIt(
+                        self.__cleanEscape(oneLine)
+                    )
 
                 else:
                     raise ReaderError('Unknown mode "{0}".'.format(mode))
 
 
-# ONLY ONE CONTENT OF LINES OR A BLOCK
-        if not oneBlockHasBeenFound:
+# -- ONLY A CONTENT OF LINES OR A BLOCK -- #
+
+        if not blockFound:
 # List for the line mode (verbatim)
             if mode == "line":
                 listLines = []
@@ -549,15 +662,169 @@ This method does all the ugly job for analysing the ``orpyste`` file.
 
                 return (firstLineNb, blockContent)
 
+
+# -- ALL THE JOB HAS BEEN DONE ! -- #
+
 # Return the ordered dictionary.
         return answer
 
+
+# --------------------- #
+# -- FLAT DICTIONARY -- #
+# --------------------- #
+
+    def __flattenDict(self, oneDict):
+        """
+-----------------
+Small description
+-----------------
+
+This method transforms the ``(Number line, Content)`` dictionary to a simple
+``Content`` dictionary.
+        """
+        flatDict = collections.OrderedDict()
+
+        for oneKey, oneValue in oneDict.items():
+            if isinstance(oneValue, dict) \
+            or isinstance(oneValue, collections.OrderedDict):
+                oneValue = self.__flattenDict(oneValue)
+
+            elif isinstance(oneValue, tuple):
+                oneValue = oneValue[1]
+
+            elif isinstance(oneValue, list):
+                oneValue = [x[1] for x in oneValue]
+
+            flatDict[oneKey] = oneValue
+
+        return flatDict
+
+
+# ---------------------------- #
+# -- EQUAL (PLUS) LIKE MODE -- #
+# ---------------------------- #
+
+    def __findOnSep(self, text, sep):
+        """
+-----------------
+Small description
+-----------------
+
+This methods returns the position in ``text`` of the first string ``sep`` which
+is not preceded by a backslash.
+
+If there is no matchning, the value ``(-1)`` is returned.
+        """
+        i = text.find(sep)
+        j = text.find('\\' + sep)
+
+        if i == j + 1:
+            i = -1
+
+        return i
+
+    def __findSep(self, text):
+        """
+-----------------
+Small description
+-----------------
+
+This method returns the possible position of the first not esacaped separator
+(this is for for the the modes ``"equal"``, ``"equal+"``, ``"keyval"`` and
+``"keyval+"``). Indeed, the method returns ``(iKey, iVal, sep)`` where each
+parameter has the following meaning.
+
+    1) ``iKey`` is the position where the separator has been found.
+
+    2) ``iVal`` is the position of the end of the separator.
+
+    3) ``sep`` is the separator found.
+        """
+        iKey = -1
+        iVal = -1
+        sep  = None
+
+        for oneSep in self.sep:
+            j = self.__findOnSep(
+                text = text,
+                sep  = oneSep
+            )
+
+            if j != -1 and (iKey == -1 or j < iKey):
+                iKey = j
+                iVal = iKey + len(oneSep)
+                sep  = oneSep
+
+        return iKey, iVal, sep
+
+    def __cleanEscape(
+        self,
+        text,
+        escapeKeyval = False
+    ):
+        """
+-----------------
+Small description
+-----------------
+
+This methods simply removes backslashes that escapes double points.
+
+If ``escapeKeyval = True``, then there is alos a cleaning of the backslashes
+escaping separators for the the modes ``"equal"``, ``"equal+"``, ``"keyval"``
+and ``"keyval+"``.
+
+By default, ``escapeKeyval = False``.
+        """
+        text = text.replace('\::', '::')
+
+        if escapeKeyval:
+            for oneSep in self.sep:
+                text = text.replace('\\' + oneSep, oneSep)
+
+        return text
+
+    def __storeKeyval(
+        self,
+        answer,
+        key,
+        val,
+        sep,
+        mode
+    ):
+        """
+-----------------
+Small description
+-----------------
+
+This methods manages the storing for values for the modes ``"equal"``,
+``"equal+"``, ``"keyval"`` and ``"keyval+"``.
+        """
+        if mode.startswith("keyval"):
+            subCategory = answer.get(sep, {})
+
+            subCategory[key] = self.__cleanEscape(
+                text         = val,
+                escapeKeyval = True
+            )
+
+            answer[sep] = subCategory
+
+        else:
+            answer[key] = self.__cleanEscape(
+                text         = val,
+                escapeKeyval = True
+            )
+
 # -------------------- #
 # -- SENDING ERRORS -- #
 # -------------------- #
 
     def __sendError(self, message):
         """
+-----------------
+Small description
+-----------------
+
 This method simply eases the raising of errors by adding infos about the file or
 the string content analysed.
         """
